@@ -13,7 +13,7 @@
 #   limitations under the License.
 import warnings
 
-from collections.abc import Callable, Generator, Iterable, Sequence
+from collections.abc import Iterable, Sequence
 from typing import cast
 
 import numpy as np
@@ -32,20 +32,20 @@ from pytensor.graph.basic import (
     Variable,
     clone_get_equiv,
     equal_computations,
-    graph_inputs,
-    walk,
 )
 from pytensor.graph.fg import FunctionGraph, Output
+from pytensor.graph.op import HasInnerGraph
+from pytensor.graph.traversal import explicit_graph_inputs, graph_inputs, walk
 from pytensor.scalar.basic import Cast
 from pytensor.scan.op import Scan
 from pytensor.tensor.basic import _as_tensor_variable
 from pytensor.tensor.elemwise import Elemwise
-from pytensor.tensor.random.op import RandomVariable
+from pytensor.tensor.random.op import RandomVariable, RNGConsumerOp
 from pytensor.tensor.random.type import RandomType
 from pytensor.tensor.random.var import RandomGeneratorSharedVariable
 from pytensor.tensor.rewriting.basic import topo_unconditional_constant_folding
 from pytensor.tensor.rewriting.shape import ShapeFeature
-from pytensor.tensor.sharedvar import SharedVariable, TensorSharedVariable
+from pytensor.tensor.sharedvar import SharedVariable
 from pytensor.tensor.subtensor import AdvancedIncSubtensor, AdvancedIncSubtensor1
 from pytensor.tensor.variable import TensorVariable
 
@@ -55,11 +55,9 @@ from pymc.vartypes import continuous_types, isgenerator, typefilter
 
 PotentialShapeType = int | np.ndarray | Sequence[int | Variable] | TensorVariable
 
-
 __all__ = [
     "CallableTensor",
     "compile",
-    "compile_pymc",
     "cont_inputs",
     "convert_data",
     "convert_observed_data",
@@ -136,6 +134,9 @@ def dataframe_to_tensor_variable(df: pd.DataFrame, *args, **kwargs) -> TensorVar
     return pt.as_tensor_variable(df.to_numpy(), *args, **kwargs)
 
 
+_cheap_eval_mode = Mode(linker="py", optimizer="minimum_compile")
+
+
 def extract_obs_data(x: TensorVariable) -> np.ndarray:
     """Extract data from observed symbolic variables.
 
@@ -164,45 +165,29 @@ def extract_obs_data(x: TensorVariable) -> np.ndarray:
             mask[mask_idx] = 1
             return np.ma.MaskedArray(array_data, mask)
 
-    from pymc.logprob.utils import rvs_in_graph
-
-    if not inputvars(x) and not rvs_in_graph(x):
-        cheap_eval_mode = Mode(linker="py", optimizer=None)
-        return x.eval(mode=cheap_eval_mode)
+    if not len(list(explicit_graph_inputs(x))) and not rvs_in_graph(x):
+        return x.eval(mode=_cheap_eval_mode)
 
     raise TypeError(f"Data cannot be extracted from {x}")
 
 
-def walk_model(
-    graphs: Iterable[TensorVariable],
-    stop_at_vars: set[TensorVariable] | None = None,
-    expand_fn: Callable[[TensorVariable], Iterable[TensorVariable]] = lambda var: [],
-) -> Generator[TensorVariable, None, None]:
-    """Walk model graphs and yield their nodes.
+def expand_inner_graph(r):
+    if (node := r.owner) is not None:
+        inputs = list(reversed(node.inputs))
 
-    Parameters
-    ----------
-    graphs
-        The graphs to walk.
-    stop_at_vars
-        A list of variables at which the walk will terminate.
-    expand_fn
-        A function that returns the next variable(s) to be traversed.
-    """
-    warnings.warn("walk_model will be removed in a future relase of PyMC", FutureWarning)
+        if isinstance(node.op, HasInnerGraph):
+            inputs += node.op.inner_outputs
 
-    if stop_at_vars is None:
-        stop_at_vars = set()
+        return inputs
 
-    def expand(var):
-        new_vars = expand_fn(var)
 
-        if var.owner and var not in stop_at_vars:
-            new_vars.extend(reversed(var.owner.inputs))
-
-        return new_vars
-
-    yield from walk(graphs, expand, bfs=False)
+def rvs_in_graph(vars: Variable | Sequence[Variable], rv_ops=None) -> set[Variable]:
+    """Assert that there are no random nodes in a graph."""
+    return {
+        var
+        for var in walk(makeiter(vars), expand_inner_graph, False)
+        if (var.owner and isinstance(var.owner.op, RNGConsumerOp))
+    }
 
 
 def replace_vars_in_graphs(
@@ -259,15 +244,17 @@ def cont_inputs(a):
     """
     Get the continuous inputs into PyTensor variables.
 
+    NOTE: No particular order is guaranteed across PyTensor versions
+
     Parameters
     ----------
         a: PyTensor variable
 
     Returns
     -------
-        r: list of tensor variables that are continuous inputs
+        r: list of tensor variables that are continuous inputs.
     """
-    return typefilter(inputvars(a), continuous_types)
+    return typefilter(explicit_graph_inputs(a), continuous_types)
 
 
 def floatX(X):
@@ -314,7 +301,9 @@ PyTensor derivative functions
 
 def gradient1(f, v):
     """Flat gradient of f wrt v."""
-    return pt.flatten(grad(f, v, disconnected_inputs="warn"))
+    return pt.as_tensor(
+        grad(f, v, disconnected_inputs="warn"), allow_xtensor_conversion=True
+    ).ravel()
 
 
 empty_gradient = pt.zeros(0, dtype="float32")
@@ -323,6 +312,10 @@ empty_gradient = pt.zeros(0, dtype="float32")
 def gradient(f, vars=None):
     if vars is None:
         vars = cont_inputs(f)
+        if len(vars) > 1:
+            raise ValueError(
+                "gradient requires vars to be specified when there is more than one input."
+            )
 
     if vars:
         return pt.concatenate([gradient1(f, v) for v in vars], axis=0)
@@ -338,12 +331,16 @@ def jacobian1(f, v):
     def grad_i(i):
         return gradient1(f[i], v)
 
-    return pytensor.map(grad_i, idx)[0]
+    return pytensor.map(grad_i, idx, return_updates=False)
 
 
 def jacobian(f, vars=None):
     if vars is None:
         vars = cont_inputs(f)
+        if len(vars) > 1:
+            raise ValueError(
+                "jacobian requires vars to be specified when there is more than one input."
+            )
 
     if vars:
         return pt.concatenate([jacobian1(f, v) for v in vars], axis=1)
@@ -358,8 +355,13 @@ def jacobian_diag(f, x):
         return grad(f[i], x)[i]
 
     return pytensor.scan(
-        grad_ii, sequences=[idx], n_steps=f.shape[0], non_sequences=[f, x], name="jacobian_diag"
-    )[0]
+        grad_ii,
+        sequences=[idx],
+        n_steps=f.shape[0],
+        non_sequences=[f, x],
+        name="jacobian_diag",
+        return_updates=False,
+    )
 
 
 @pytensor.config.change_flags(compute_test_value="ignore")
@@ -384,13 +386,17 @@ def hessian_diag1(f, v):
     def hess_ii(i):
         return gradient1(g[i], v)[i]
 
-    return pytensor.map(hess_ii, idx)[0]
+    return pytensor.map(hess_ii, idx, return_updates=False)
 
 
 @pytensor.config.change_flags(compute_test_value="ignore")
 def hessian_diag(f, vars=None, negate_output=True):
     if vars is None:
         vars = cont_inputs(f)
+        if len(vars) > 1:
+            raise ValueError(
+                "hessian_diag requires vars to be specified when there is more than one input."
+            )
 
     if vars:
         res = pt.concatenate([hessian_diag1(f, v) for v in vars], axis=0)
@@ -424,20 +430,21 @@ def make_shared_replacements(point, vars, model):
     -------
     Dict of variable -> new shared variable
     """
-    othervars = set(model.value_vars) - set(vars)
+    vars_set = set(vars)
     return {
         var: pytensor.shared(point[var.name], var.name + "_shared", shape=var.type.shape)
-        for var in othervars
+        for var in model.value_vars
+        if var not in vars_set
     }
 
 
 def join_nonshared_inputs(
     point: dict[str, np.ndarray],
-    outputs: list[TensorVariable],
-    inputs: list[TensorVariable],
-    shared_inputs: dict[TensorVariable, TensorSharedVariable] | None = None,
+    outputs: Sequence[Variable],
+    inputs: Sequence[Variable],
+    shared_inputs: dict[Variable, Variable] | None = None,
     make_inputs_shared: bool = False,
-) -> tuple[list[TensorVariable], TensorVariable]:
+) -> tuple[Sequence[Variable], TensorVariable]:
     """
     Create new outputs and input TensorVariables where the non-shared inputs are joined in a single raveled vector input.
 
@@ -562,7 +569,9 @@ def join_nonshared_inputs(
     if not inputs:
         raise ValueError("Empty list of input variables.")
 
-    raveled_inputs = pt.concatenate([var.ravel() for var in inputs])
+    raveled_inputs = pt.concatenate(
+        [pt.as_tensor(var, allow_xtensor_conversion=True).ravel() for var in inputs]
+    )
 
     if not make_inputs_shared:
         tensor_type = raveled_inputs.type
@@ -574,12 +583,15 @@ def join_nonshared_inputs(
     if pytensor.config.compute_test_value != "off":
         joined_inputs.tag.test_value = raveled_inputs.tag.test_value
 
-    replace: dict[TensorVariable, TensorVariable] = {}
+    replace: dict[Variable, Variable] = {}
     last_idx = 0
     for var in inputs:
         shape = point[var.name].shape
         arr_len = np.prod(shape, dtype=int)
-        replace[var] = joined_inputs[last_idx : last_idx + arr_len].reshape(shape).astype(var.dtype)
+        replacement_var = (
+            joined_inputs[last_idx : last_idx + arr_len].reshape(shape).astype(var.dtype)
+        )
+        replace[var] = var.type.filter_variable(replacement_var)
         last_idx += arr_len
 
     if shared_inputs is not None:
@@ -594,12 +606,17 @@ def join_nonshared_inputs(
 class PointFunc:
     """Wraps so a function so it takes a dict of arguments instead of arguments."""
 
+    __slots__ = ("f",)
+
     def __init__(self, f):
         self.f = f
 
     def __call__(self, state):
         return self.f(**state)
 
+    def dprint(self, **kwrags):
+        return self.f.dprint(**kwrags)
+
 
 class CallableTensor:
     """Turns a symbolic variable with one input into a function that returns symbolic arguments with the one variable replaced with the input."""
@@ -614,7 +631,7 @@ class CallableTensor:
         ----------
         input: TensorVariable
         """
-        (oldinput,) = inputvars(self.tensor)
+        (oldinput,) = explicit_graph_inputs(self.tensor)
         return pytensor.clone_replace(self.tensor, {oldinput: input}, rebuild_strict=False)
 
 
@@ -724,17 +741,19 @@ def collect_default_updates(
 
     Examples
     --------
-    .. code:: python
+    .. code-block:: python
 
         import pymc as pm
         from pytensor.scan import scan
         from pymc.pytensorf import collect_default_updates
 
+
         def scan_step(xtm1):
             x = xtm1 + pm.Normal.dist()
             x_update = collect_default_updates([x])
             return x, x_update
 
+
         x0 = pm.Normal.dist()
 
         xs, updates = scan(
@@ -748,8 +767,6 @@ def collect_default_updates(
         xs_draws = pm.draw(xs, draws=10)
 
     """
-    # Avoid circular import
-    from pymc.distributions.distribution import SymbolicRandomVariable
 
     def find_default_update(clients, rng: Variable) -> None | Variable:
         rng_clients = clients.get(rng, None)
@@ -792,48 +809,47 @@ def collect_default_updates(
         [client, _] = rng_clients[0]
 
         # RNG is an output of the function, this is not a problem
-        if isinstance(client.op, Output):
-            return None
+        client_op = client.op
 
-        # RNG is used by another operator, which should output an update for the RNG
-        if isinstance(client.op, RandomVariable):
-            # RandomVariable first output is always the update of the input RNG
-            next_rng = client.outputs[0]
-
-        elif isinstance(client.op, SymbolicRandomVariable):
-            # SymbolicRandomVariable have an explicit method that returns an
-            # update mapping for their RNG(s)
-            next_rng = client.op.update(client).get(rng)
-            if next_rng is None:
-                raise ValueError(
-                    f"No update found for at least one RNG used in SymbolicRandomVariable Op {client.op}"
-                )
-        elif isinstance(client.op, Scan):
-            # Check if any shared output corresponds to the RNG
-            rng_idx = client.inputs.index(rng)
-            io_map = client.op.get_oinp_iinp_iout_oout_mappings()["outer_out_from_outer_inp"]
-            out_idx = io_map.get(rng_idx, -1)
-            if out_idx != -1:
-                next_rng = client.outputs[out_idx]
-            else:  # No break
-                raise ValueError(
-                    f"No update found for at least one RNG used in Scan Op {client.op}.\n"
-                    "You can use `pytensorf.collect_default_updates` inside the Scan function to return updates automatically."
-                )
-        elif isinstance(client.op, OpFromGraph):
-            try:
-                next_rng = collect_default_updates_inner_fgraph(client).get(rng)
+        match client_op:
+            case Output():
+                return None
+            # Otherwise, RNG is used by another operator, which should output an update for the RNG
+            case RandomVariable():
+                # RandomVariable first output is always the update of the input RNG
+                next_rng = client.outputs[0]
+            case RNGConsumerOp():
+                # RNGConsumerOp have an explicit method that returns an update mapping for their RNG(s)
+                # RandomVariable is a subclass of RNGConsumerOp, but we specialize above for speedup
+                next_rng = client_op.update(client).get(rng)
                 if next_rng is None:
-                    # OFG either does not make use of this RNG or inconsistent use that will have emitted a warning
-                    return None
-            except ValueError as exc:
-                raise ValueError(
-                    f"No update found for at least one RNG used in OpFromGraph Op {client.op}.\n"
-                    "You can use `pytensorf.collect_default_updates` and include those updates as outputs."
-                ) from exc
-        else:
-            # We don't know how this RNG should be updated. The user should provide an update manually
-            return None
+                    raise ValueError(f"No update found for at least one RNG used in {client_op}")
+            case Scan():
+                # Check if any shared output corresponds to the RNG
+                rng_idx = client.inputs.index(rng)
+                io_map = client_op.get_oinp_iinp_iout_oout_mappings()["outer_out_from_outer_inp"]
+                out_idx = io_map.get(rng_idx, -1)
+                if out_idx != -1:
+                    next_rng = client.outputs[out_idx]
+                else:  # No break
+                    raise ValueError(
+                        f"No update found for at least one RNG used in Scan Op {client_op}.\n"
+                        "You can use `pytensorf.collect_default_updates` inside the Scan function to return updates automatically."
+                    )
+            case OpFromGraph():
+                try:
+                    next_rng = collect_default_updates_inner_fgraph(client).get(rng)
+                    if next_rng is None:
+                        # OFG either does not make use of this RNG or inconsistent use that will have emitted a warning
+                        return None
+                except ValueError as exc:
+                    raise ValueError(
+                        f"No update found for at least one RNG used in OpFromGraph Op {client_op}.\n"
+                        "You can use `pytensorf.collect_default_updates` and include those updates as outputs."
+                    ) from exc
+            case _:
+                # We don't know how this RNG should be updated. The user should provide an update manually
+                return None
 
         # Recurse until we find final update for RNG
         nested_next_rng = find_default_update(clients, next_rng)
@@ -954,14 +970,6 @@ def compile(
     return pytensor_function
 
 
-def compile_pymc(*args, **kwargs):
-    warnings.warn(
-        "compile_pymc was renamed to compile. Old name will be removed in a future release of PyMC",
-        FutureWarning,
-    )
-    return compile(*args, **kwargs)
-
-
 def constant_fold(
     xs: Sequence[TensorVariable], raise_not_constant: bool = True
 ) -> tuple[np.ndarray | Variable, ...]:
@@ -1030,43 +1038,16 @@ def as_symbolic_string(x, **kwargs):
 
 
 def toposort_replace(
-    fgraph: FunctionGraph, replacements: Sequence[tuple[Variable, Variable]], reverse: bool = False
+    fgraph: FunctionGraph,
+    replacements: Sequence[tuple[Variable, Variable]],
+    reverse: bool = False,
 ) -> None:
     """Replace multiple variables in place in topological order."""
     fgraph_toposort = {node: i for i, node in enumerate(fgraph.toposort())}
-    _inner_fgraph_toposorts = {}  # Cache inner toposorts
-
-    def _nested_toposort_index(var, fgraph_toposort) -> tuple[int]:
-        """Compute position of variable in fgraph toposort.
-
-        When a variable is an OpFromGraph output, extend output with the toposort index of the inner graph(s).
-
-        This allows ordering variables that come from the same OpFromGraph.
-        """
-        if not var.owner:
-            return (-1,)
-
-        index = fgraph_toposort[var.owner]
-
-        # Recurse into OpFromGraphs
-        # TODO: Could also recurse into Scans
-        if isinstance(var.owner.op, OpFromGraph):
-            inner_fgraph = var.owner.op.fgraph
-
-            if inner_fgraph not in _inner_fgraph_toposorts:
-                _inner_fgraph_toposorts[inner_fgraph] = {
-                    node: i for i, node in enumerate(inner_fgraph.toposort())
-                }
-
-            inner_fgraph_toposort = _inner_fgraph_toposorts[inner_fgraph]
-            inner_var = inner_fgraph.outputs[var.owner.outputs.index(var)]
-            return (index, *_nested_toposort_index(inner_var, inner_fgraph_toposort))
-        else:
-            return (index,)
-
+    fgraph_toposort[None] = -1  # Variables without owner are not in the toposort
     sorted_replacements = sorted(
         replacements,
-        key=lambda pair: _nested_toposort_index(pair[0], fgraph_toposort),
+        key=lambda pair: fgraph_toposort[pair[0].owner],
         reverse=reverse,
     )
     fgraph.replace_all(sorted_replacements, import_missing=True)
